<?php
/**
 * ETidyCleanRepair class file.
 *
 * @author MetaYii
 * @link http://www.yiiframework.com/
 * @copyright Copyright &copy; 2008 MetaYii
 * @license
 *
 * Copyright © 2008 by MetaYii
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or other
 *   materials provided with the distribution.
 * - Neither the name of Yii Software LLC nor the names of its contributors may be used
 *   to endorse or promote products derived from this software without specific prior
 *   written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/**
 * ETidyCleanRepair is wrapper of {@link http://www.php.net/manual/es/book.tidy.php Tidy}.
 *
 * Tidy is a binding for the Tidy HTML clean and repair utility which allows you to not
 * only clean and otherwise manipulate HTML documents, but also traverse the document tree.
 * To use Tidy, you will need libtidy installed, available on the {@link http://tidy.sourceforge.net/ tidy homepage}.
 * In PHP 5 you need only to compile using the --with-tidy option
 *
 * ETidyCleanRepair can be used as either a widget or a controller filter.
 *
 *
 * @author MetaYii
 * @package application.extensions.tidycleanrepair
 * @since 1.0
 */
class ETidyCleanRepair extends COutputProcessor
{
	/**
	 * @var mixed the configuration for {@link http://www.php.net/manual/es/tidy.examples.php Tidy}.
	 * This can be an array of directives (Option => Value) or the path to an ini file. The defaults
	 * are very basic, just clean the code and return only the body, not a full HTML page.
	 * @see http://tidy.sourceforge.net/docs/quickref.html
	 */
	private $config = array('clean'=>true, 'show-body-only'=>true);

	/**
	 * @var string The encoding parameter sets the encoding for input/output documents. The possible
	 * values for encoding  are: ascii, latin0, latin1, raw, utf8, iso2022, mac, win1252, ibm858,
	 * utf16, utf16le, utf16be, big5 and shiftjis.
	 */
	private $encoding = 'utf8';

	/**
	 * Sets the encoding property
	 *
	 * @param string $value
	 */
	public function setEncoding($value)
	{
	   $v = strtolower($value);
	   if (!in_array($v, array('ascii', 'latin0', 'latin1', 'raw', 'utf8', 'iso2022', 'mac', 'win1252', 'ibm858', 'utf16', 'utf16le', 'utf16be', 'big5', 'shiftjis')))
         throw new CException(Yii::t('ETidyCleanRepair', '"{value}" is not a valid encoding.', array('{value}'=>$value)));
      $this->encoding = $v;
	}

	/**
	 * Gets the encoding property
	 *
	 * @return string
	 */
	public function getEncoding()
	{
	   return $this->encoding;
	}

	/**
	 * Sets the configuration for Tidy. This can be an array with the configuration pairs Option => Value,
	 * or a string with the path of a valid configuration file.
	 * @see http://tidy.sourceforge.net/docs/quickref.html
	 *
	 * @param mixed $value
	 */
	public function setConfig($value)
	{
	   if (!is_array($value) && !is_string($value))
         throw new CException(Yii::t('ETidyCleanRepair', '"{value}" is not a string nor an array. Invalid configuration.', array('{value}'=>$value)));
      if (is_string($value) && !is_readable($value))
         throw new CException(Yii::t('ETidyCleanRepair', 'Cannot read Tidy configuration file.'));
      $this->config = $value;
	}

	/**
	 * Gets the configuration
	 *
	 * @return mixed
	 */
	public function getConfig()
	{
	   return $this->config;
	}

	/**
	 * Processes the captured output.
    * This method process {@link http://www.php.net/manual/es/book.tidy.php Tidy}.
	 * @param string the captured output to be processed
	 */
	public function processOutput($output)
	{
		$output=$this->cleanRepair($output);
		parent::processOutput($output);
	}

	/**
	 * Execute configured cleanup and repair operations on parsed markup.
	 * @param string the content to be purified.
	 * @return string the purified content
	 */
	public function cleanRepair($content)
	{
	   if (function_exists('tidy_repair_string')) {
         return @tidy_repair_string($content, $this->config, $this->encoding);
	   }
	   return $content;
	}
}
